﻿using System;
using System.Text;
using System.Web;
using System.Web.UI.WebControls;
using System.Text.RegularExpressions;
using eBlog.Cache;
using eBlog.Common.Security;
namespace eBlog.Common
{
    /// <summary>
    /// 页面数据校验类
    /// </summary>
    public class PageValidate
    {
        private static readonly Regex RegPhone = new Regex("^[0-9]+[-]?[0-9]+[-]?[0-9]$");
        private static readonly Regex RegNumber = new Regex("^[0-9]+$");
        private static readonly Regex RegNumberSign = new Regex("^[+-]?[0-9]+$");
        private static readonly Regex RegDecimal = new Regex("^[0-9]+[.]?[0-9]+$");
        private static readonly Regex RegDecimalSign = new Regex("^[+-]?[0-9]+[.]?[0-9]+$"); //等价于^[+-]?\d+[.]?\d+$
        private static readonly Regex RegEmail = new Regex("^[\\w-]+@[\\w-]+\\.(com|net|org|edu|mil|tv|biz|info|cn)$");//w 英文字母或数字的字符串，和 [a-zA-Z0-9] 语法一样 
        private static readonly Regex RegCHZN = new Regex("[\u4e00-\u9fa5]");


        /// <summary>
        /// 检测管理员是否登录
        /// </summary>
        /// <param name="CurrPage"></param>
        public static void CheckAdminLogin(System.Web.UI.Page CurrPage)
        {

            //HttpCookie AdminCookie = new HttpCookie("AdminUser");
            //AdminCookie["name"] = "richwong";
            //AdminCookie["key"] = "key";
            //AdminCookie.Expires = DateTime.Now.AddMinutes(30);       

            if (HttpContext.Current.Request.Cookies["eBlogAdmin"] == null || (HttpContext.Current.Request.Cookies["eBlogAdmin"].Values["key"] != MD5.FormatCookie(HttpContext.Current.Request.Cookies["eBlogAdmin"].Values["name"])))
            {
                CurrPage.Response.Redirect("Login.aspx");
                return;
            }

        }

        public static void CheckUserLogin(System.Web.UI.Page CurrPage)
        {

            if (HttpContext.Current.Request.Cookies["eBlogUser"] == null || (HttpContext.Current.Request.Cookies["eBlogUser"].Values["key"] != MD5.FormatCookie(HttpContext.Current.Request.Cookies["eBlogUser"].Values["name"])))
            {
                CurrPage.Response.Redirect("/users/login.aspx?ref=" + CurrPage.Request.Url.PathAndQuery);
                return;
            }

        }


        public static void SaveCookies(System.Web.UI.Page CurrPage, string CookieName, string value, double Expires)
        {
            HttpCookie System_Cookies_ = new HttpCookie(CookieName)
                                             {
                                                 Value = value,
                                                 Domain = "",
                                                 Expires = DateTime.Now.AddHours(Expires)
                                             };

            if (HttpContext.Current.Request.Cookies[CookieName] == null)
            {
                CurrPage.Response.Cookies.Add(System_Cookies_);
            }
            else
            {
                CurrPage.Response.Cookies.Set(System_Cookies_); //更新         
            }
        }
        public static void SaveCookies(System.Web.UI.Page CurrPage, string CookieName, string value)
        {
            SaveCookies(CurrPage, CookieName, value, 24); //默认24小时
        }

        public static string GetCookies(string CookieName)
        {
            if (HttpContext.Current.Request.Cookies[CookieName] != null)
                return HttpContext.Current.Request.Cookies[CookieName].Value;
            else
                return "";
        }


        public static string Query(string param)
        {
            string qValue = "";

            if (HttpContext.Current.Request.QueryString[param] != null && HttpContext.Current.Request.QueryString[param].Trim() != "")
            {

                qValue = HttpContext.Current.Request.QueryString[param];

            }
            return qValue;

        }
        public static int QueryInt(string param)
        {
            if (Query(param) != "")
            {
                int qValue = 0;
                if (int.TryParse(Query(param), out qValue))
                    return qValue;
                else
                    return -1;
                    //MessageBox.Show("只能为数字");
            }
            return 0;


        }
        /// <summary>
        /// 转换INT类型,默认输出0
        /// </summary>
        /// <param name="param"></param>
        /// <returns></returns>
        public static int GetDefaultInt(string param)
        {
            if (!string.IsNullOrEmpty(param))
            {
                int returnValue = 0;
                if (int.TryParse(param, out returnValue))
                    return returnValue;
                else
                    return returnValue;
            }
            return 0;
        }

        public static string Form(string param)
        {
            var qValue = "";

            if (HttpContext.Current.Request.Form[param] != null && HttpContext.Current.Request.Form[param].Trim() != "")
            {

                qValue = HttpContext.Current.Request.Form[param];

            }

            return qValue;

        }



        #region 数字字符串检查
        public static bool IsPhone(string inputData)
        {
            if (string.IsNullOrEmpty(inputData))
                return false;
            Match m = RegPhone.Match(inputData);
            return m.Success;
        }
        /// <summary>
        /// 检查Request查询字符串的键值，是否是数字，最大长度限制
        /// </summary>
        /// <param name="req">Request</param>
        /// <param name="inputKey">Request的键值</param>
        /// <param name="maxLen">最大长度</param>
        /// <returns>返回Request查询字符串</returns>
        public static string FetchInputDigit(HttpRequest req, string inputKey, int maxLen)
        {
            string retVal = string.Empty;
            if (inputKey != null && inputKey != string.Empty)
            {
                retVal = req.QueryString[inputKey];
                if (null == retVal)
                    retVal = req.Form[inputKey];
                if (null != retVal)
                {
                    retVal = SqlText(retVal, maxLen);
                    if (!IsNumber(retVal))
                        retVal = string.Empty;
                }
            }
            if (retVal == null)
                retVal = string.Empty;
            return retVal;
        }
        /// <summary>
        /// 是否数字字符串
        /// </summary>
        /// <param name="inputData">输入字符串</param>
        /// <returns></returns>
        public static bool IsNumber(string inputData)
        {
            if (string.IsNullOrEmpty(inputData))
                return false;

            Match m = RegNumber.Match(inputData);
            return m.Success;
        }

        /// <summary>
        /// 是否数字字符串 可带正负号
        /// </summary>
        /// <param name="inputData">输入字符串</param>
        /// <returns></returns>
        public static bool IsNumberSign(string inputData)
        {
            Match m = RegNumberSign.Match(inputData);
            return m.Success;
        }
        /// <summary>
        /// 是否是浮点数
        /// </summary>
        /// <param name="inputData">输入字符串</param>
        /// <returns></returns>
        public static bool IsDecimal(string inputData)
        {
            if (IsNumber(inputData))
            {
                return true;
            }
            else
            {
                Match m = RegDecimal.Match(inputData);
                return m.Success;
            }
        }
        /// <summary>
        /// 是否是浮点数 可带正负号
        /// </summary>
        /// <param name="inputData">输入字符串</param>
        /// <returns></returns>
        public static bool IsDecimalSign(string inputData)
        {

            Match m = RegDecimalSign.Match(inputData);
            return m.Success;

        }

        #endregion

        #region 中文检测

        /// <summary>
        /// 检测是否有中文字符
        /// </summary>
        /// <param name="inputData"></param>
        /// <returns></returns>
        public static bool IsHasCHZN(string inputData)
        {
            Match m = RegCHZN.Match(inputData);
            return m.Success;
        }

        #endregion

        #region 邮件地址
        /// <summary>
        /// 是否是浮点数 可带正负号
        /// </summary>
        /// <param name="inputData">输入字符串</param>
        /// <returns></returns>
        public static bool IsEmail(string inputData)
        {
            if (string.IsNullOrEmpty(inputData)) return false;
            Match m = RegEmail.Match(inputData);
            return m.Success;
        }

        #endregion


        #region IP检查
        /// <summary>
        /// 是否IP
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static bool IsIP(string source)
        {
            if (source == null)
                return false;

            return Regex.IsMatch(source, @"^(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9])\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[0-9])$", RegexOptions.IgnoreCase);
        }
        #endregion

        #region 日期格式判断
        /// <summary>
        /// 日期格式字符串判断
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static bool IsDateTime(string str)
        {
            try
            {
                if (!string.IsNullOrEmpty(str))
                {
                    DateTime.Parse(str);
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch
            {
                return false;
            }
        }
        #endregion

        #region 其他

        /// <summary>
        /// 检查字符串最大长度，返回指定长度的串
        /// </summary>
        /// <param name="sqlInput">输入字符串</param>
        /// <param name="maxLength">最大长度</param>
        /// <returns></returns>			
        public static string SqlText(string sqlInput, int maxLength)
        {
            if (sqlInput != null && sqlInput != string.Empty)
            {
                sqlInput = sqlInput.Trim();
                if (sqlInput.Length > maxLength)//按最大长度截取字符串
                    sqlInput = sqlInput.Substring(0, maxLength);
            }
            return sqlInput;
        }
        /// <summary>
        /// 字符串编码
        /// </summary>
        /// <param name="inputData"></param>
        /// <returns></returns>
        public static string HtmlEncode(string inputData)
        {
            return HttpUtility.HtmlEncode(inputData);
        }
        /// <summary>
        /// 设置Label显示Encode的字符串
        /// </summary>
        /// <param name="lbl"></param>
        /// <param name="txtInput"></param>
        public static void SetLabel(Label lbl, string txtInput)
        {
            lbl.Text = HtmlEncode(txtInput);
        }
        public static void SetLabel(Label lbl, object inputObj)
        {
            SetLabel(lbl, inputObj.ToString());
        }
        //字符串清理
        public static string InputText(string inputString, int maxLength)
        {
            StringBuilder retVal = new StringBuilder();

            // 检查是否为空
            if ((inputString != null) && (inputString != string.Empty))
            {
                inputString = inputString.Trim();

                //检查长度
                if (inputString.Length > maxLength)
                    inputString = inputString.Substring(0, maxLength);

                //替换危险字符
                for (int i = 0; i < inputString.Length; i++)
                {
                    switch (inputString[i])
                    {
                        case '"':
                            retVal.Append("&quot;");
                            break;
                        case '<':
                            retVal.Append("&lt;");
                            break;
                        case '>':
                            retVal.Append("&gt;");
                            break;
                        default:
                            retVal.Append(inputString[i]);
                            break;
                    }
                }
                retVal.Replace("'", " ");// 替换单引号
            }
            return retVal.ToString();

        }
        /// <summary>
        /// 转换成 HTML code
        /// </summary>
        /// <param name="str">string</param>
        /// <returns>string</returns>
        public static string Encode(string str)
        {
            str = str.Replace("&", "&amp;");
            str = str.Replace("'", "''");
            str = str.Replace("\"", "&quot;");
            str = str.Replace(" ", "&nbsp;");
            str = str.Replace("<", "&lt;");
            str = str.Replace(">", "&gt;");
            str = str.Replace("\n", "<br>");
            return str;
        }
        /// <summary>
        ///解析html成 普通文本
        /// </summary>
        /// <param name="str">string</param>
        /// <returns>string</returns>
        public static string Decode(string str)
        {
            str = str.Replace("<br>", "\n");
            str = str.Replace("&gt;", ">");
            str = str.Replace("&lt;", "<");
            str = str.Replace("&nbsp;", " ");
            str = str.Replace("&quot;", "\"");
            return str;
        }

        public static string SqlTextClear(string sqlText)
        {
            if (sqlText == null)
            {
                return null;
            }
            if (sqlText == "")
            {
                return "";
            }
            sqlText = sqlText.Replace(",", "");//去除,
            sqlText = sqlText.Replace("<", "");//去除<
            sqlText = sqlText.Replace(">", "");//去除>
            sqlText = sqlText.Replace("--", "");//去除--
            sqlText = sqlText.Replace("'", "");//去除'
            sqlText = sqlText.Replace("\"", "");//去除"
            sqlText = sqlText.Replace("=", "");//去除=
            sqlText = sqlText.Replace("%", "");//去除%
            sqlText = sqlText.Replace(" ", "");//去除空格
            return sqlText;
        }
        #endregion

        #region 是否由特定字符组成
        public static bool isContainSameChar(string strInput)
        {
            string charInput = string.Empty;
            if (!string.IsNullOrEmpty(strInput))
            {
                charInput = strInput.Substring(0, 1);
            }
            return isContainSameChar(strInput, charInput, strInput.Length);
        }

        public static bool isContainSameChar(string strInput, string charInput, int lenInput)
        {
            if (string.IsNullOrEmpty(charInput))
            {
                return false;
            }
            else
            {
                Regex RegNumber = new Regex(string.Format("^([{0}])+$", charInput));
                //Regex RegNumber = new Regex(string.Format("^([{0}]{{1}})+$", charInput,lenInput));
                Match m = RegNumber.Match(strInput);
                return m.Success;
            }
        }
        #endregion

        #region 检查输入的参数是不是某些定义好的特殊字符：这个方法目前用于密码输入的安全检查
        /// <summary>
        /// 检查输入的参数是不是某些定义好的特殊字符：这个方法目前用于密码输入的安全检查
        /// </summary>
        public static bool isContainSpecChar(string strInput)
        {
            string[] list = new string[] { "123456", "654321" };
            bool result = new bool();
            for (int i = 0; i < list.Length; i++)
            {
                if (strInput == list[i])
                {
                    result = true;
                    break;
                }
            }
            return result;
        }
        #endregion
    }
}
