﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using UrlRouter.Configuration;

namespace UrlRouter.Web
{
    public class RegExRewriteRule : RewriteRule
    {
        private Regex regex;

        public override void Initialize(RewriteSettings rewriteSettings)
        {
            base.Initialize(rewriteSettings);
            this.RegexOptions = rewriteSettings.GetEnumAttribute<RegexOptions>("regexOptions", RegexOptions.None);
            this.VirtualUrl = rewriteSettings.GetAttribute("virtualUrl", "");
            this.destinationUrl = rewriteSettings.GetAttribute("toUrl", "");
        }

        private void CreateRegEx()
        {
            UrlHelper urlHelper = new UrlHelper();
            if (IgnoreCase)
            {
                this.regex = new Regex(urlHelper.HandleRootOperator(virtualUrl), RegexOptions.IgnoreCase | RegexOptions.Compiled | regexOptions);
            }
            else
            {
                this.regex = new Regex(urlHelper.HandleRootOperator(virtualUrl), RegexOptions.Compiled | regexOptions);
            }
        }

        private string virtualUrl = string.Empty;

        public string VirtualUrl
        {
            get { return virtualUrl; }
            set
            {
                virtualUrl = value;
                CreateRegEx();
            }
        }
        private string destinationUrl = string.Empty;

        public string DestinationUrl
        {
            get { return destinationUrl; }
            set { destinationUrl = value; }
        }
        private RegexOptions regexOptions = RegexOptions.None;

        public RegexOptions RegexOptions
        {
            get { return regexOptions; }
            set
            {
                regexOptions = value;
                CreateRegEx();
            }
        }

        /// <summary>
        /// 能够匹配重写规则
        /// </summary>
        /// <param name="requestUrl"></param>
        /// <returns></returns>
        public override bool IsRewrite(string requestUrl)
        {
            return this.regex.IsMatch(requestUrl);
        }

        public override string RewriteUrl(string url)
        {
            return this.regex.Replace(url, this.destinationUrl, 1);
        }

    }
}
