﻿using System;
using System.Collections.Generic;
using System.Text;
using UrlRouter.Configuration.Provider;
using System.Web.Configuration;
using UrlRouter.Configuration;
using System.Configuration;
using System.Web;
using System.Collections.Specialized;

namespace UrlRouter.Web
{
    static public class UrlRewriting
    {
        static UrlRewritingProviderCollection providers = null;

        static bool initialized = false;

        static private void Initialize()
        {
            if (!initialized)
            {
                providers = new UrlRewritingProviderCollection();
                ProvidersHelper.InstantiateProviders(Configuration.Providers, providers, typeof(UrlRewritingProvider));
                if (providers["RegEx"] == null)
                {
                    RegExUrlRewritingProvider prov = new RegExUrlRewritingProvider();
                    prov.Initialize("RegEx", new NameValueCollection());
                    providers.Add(prov);
                }

                if (providers[Configuration.DefaultProvider] == null)
                {
                    string msg = string.Format("Missing the DefaultProvider {0} in the list of providers for UrlRewritingNet", Configuration.DefaultProvider);
                    throw new ApplicationException(msg);
                }
                initialized = true;
            }
        }
        /// <summary>
        /// 规则配置元素属性组
        /// </summary>
        static private UrlRewriteSection configuration;
        static private object configurationLock = new object();

        /// <summary>
        /// 读取并返回一个URL重写配置元素
        /// </summary>
        static public UrlRewriteSection Configuration
        {
            get
            {
                if (configuration == null)
                {
                    lock (configurationLock)
                    {
                        if (configuration == null)
                        {
                            UrlRewriteSection tmpConf = (UrlRewriteSection)ConfigurationManager.GetSection("urlRouter");
                            if (tmpConf == null)
                            {
                                string msg = string.Format("配置文件中缺少URL路由器元素\"urlRouter\"");
                                throw new ApplicationException(msg);
                            }
                            configuration = tmpConf;
                        }
                    }
                }
                return configuration;
            }
        }

        static public UrlRewritingProviderCollection Providers
        {
            get
            {
                Initialize();
                return providers;
            }
        }

        static public RewriteRule CreateRewriteRule()
        {
            return CreateRewriteRule(Configuration.DefaultProvider);
        }
        public static RewriteRule CreateRewriteRule(string providerName)
        {
            if (providerName == null)
                throw new ArgumentNullException("没有提供UrlRewrite的provider名字","providerName");

            UrlRewritingProvider provider = Providers[providerName];
            if (provider == null)
            {
                string msg = string.Format("未知无效的 UrlRewritingProvider {0} in list of rules", providerName);
                throw new ArgumentException(msg, "providerName");
            }
            return provider.CreateRewriteRule();
        }
        /// <summary>
        /// 添加重写规则
        /// </summary>
        /// <param name="ruleName">规则唯一命名</param>
        /// <param name="rewriteRule">规则实体</param>
        public static void AddRewriteRule(string ruleName, RewriteRule rewriteRule)
        {
            if (rewriteRule == null)
                throw new ArgumentNullException("rewriteRule"); //规则实体无效
            rewriteRule.Name = ruleName;
            HttpModuleCollection modules = System.Web.HttpContext.Current.ApplicationInstance.Modules;
            foreach (string moduleName in modules)
            {
                UrlRewriteModule rewriteModule = modules[moduleName] as UrlRewriteModule;
                if (rewriteModule != null)
                {
                    rewriteModule.AddRewriteRuleInternal(rewriteRule);
                }
            }
        }
        /// <summary>
        /// 移除重写规则
        /// </summary>
        /// <param name="ruleName"></param>
        public static void RemoveRewriteRule(string ruleName)
        {
            HttpModuleCollection modules = System.Web.HttpContext.Current.ApplicationInstance.Modules;
            foreach (string moduleName in modules)
            {
                UrlRewriteModule rewriteModule = modules[moduleName] as UrlRewriteModule;
                if (rewriteModule != null)
                {
                    rewriteModule.RemoveRewriteRuleInternal(ruleName);
                }
            }
        }
        /// <summary>
        /// 替换重写规则
        /// </summary>
        /// <param name="ruleName"></param>
        /// <param name="rewriteRule"></param>
        public static void ReplaceRewriteRule(string ruleName, RewriteRule rewriteRule)
        {
            if (rewriteRule == null)
                throw new ArgumentNullException("rewriteRule");
            rewriteRule.Name = ruleName;
            HttpModuleCollection modules = System.Web.HttpContext.Current.ApplicationInstance.Modules;
            foreach (string moduleName in modules)
            {
                UrlRewriteModule rewriteModule = modules[moduleName] as UrlRewriteModule;
                if (rewriteModule != null)
                {
                    rewriteModule.ReplaceRewriteRuleInternal(ruleName, rewriteRule);
                }
            }
        }
        /// <summary>
        /// 插入
        /// </summary>
        /// <param name="positionRuleName"></param>
        /// <param name="ruleName"></param>
        /// <param name="rewriteRule"></param>
        public static void InsertRewriteRule(string positionRuleName, string ruleName, RewriteRule rewriteRule)
        {
            if (rewriteRule == null)
                throw new ArgumentNullException("rewriteRule");
            rewriteRule.Name = ruleName;
            HttpModuleCollection modules = System.Web.HttpContext.Current.ApplicationInstance.Modules;
            foreach (string moduleName in modules)
            {
                UrlRewriteModule rewriteModule = modules[moduleName] as UrlRewriteModule;
                if (rewriteModule != null)
                {
                    rewriteModule.InsertRewriteRuleBeforeInternal( positionRuleName, ruleName, rewriteRule);
                }
            }
        }

    }
}
